package money.persistence;

import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import money.model.*;

public class MyContiReader implements ContiReader
{
	public Collection<Conto> readAll(Reader reader) throws IOException, InvalidDataException
	{
		ArrayList<Conto> conti = new ArrayList<Conto>();
		BufferedReader bufferedReader = new BufferedReader(reader);

		try
		{
			String line;
			while ((line = bufferedReader.readLine()) != null)
			{
				Conto c = null;
				StringTokenizer tok = new StringTokenizer(line, " \t");
				// il primo token lo butto via: #define
				if (!tok.nextToken().trim().equals("#define"))
					throw new InvalidDataException("Atteso #define");

				String nome = tok.nextToken().trim();
				String tipo = tok.nextToken().trim();
				if (tipo.equalsIgnoreCase("liquidi"))
				{
					c = new Liquidi(nome);
				}
				else
				{
					double value = Double.parseDouble(tok.nextToken().trim());
					if (tipo.equalsIgnoreCase("contocorrente"))
					{
						c = new ContoCorrente(nome, value);
					}
					else
					{
						if (tipo.equalsIgnoreCase("cartadicredito"))
						{
							c = new CartaCredito(nome, value);
						}
						else
							throw new InvalidDataException("Formato file errato");
					}
				}
				conti.add(c);
			}
		}
		catch (NumberFormatException e)
		{
			throw new InvalidDataException("Formato file errato");
		}
		catch (NoSuchElementException e)
		{
			throw new InvalidDataException("Formato file errato");
		}
		finally
		{
			bufferedReader.close();
		}

		return conti;

	}

}
