package money.ui;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JOptionPane;

import money.model.Bilancio;
import money.model.BilancioFamiliare;
import money.model.Conto;
import money.model.Movimento;
import money.persistence.ContiReader;
import money.persistence.MovimentiManager;

public class MyController implements Controller
{
	private BilancioFamiliare bilancio;
	private MovimentiManager movimentiManager;
	private ContiReader contiReader;

	public MyController(ContiReader contiReader, MovimentiManager movimentiManager)
	{
		this.contiReader = contiReader;
		this.movimentiManager = movimentiManager;
	}

	public void save()
	{
		try
		{
			FileOutputStream outputStream = new FileOutputStream("Bilancio.dat");
			movimentiManager.scriviMovimenti(bilancio.getMovimenti(), outputStream);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, "Errore di I/O - impossibile salvare i dati " + e.getMessage());
		}
	}

	public boolean load()
	{
		Collection<Conto> conti;
		try
		{
			FileReader reader = new FileReader("StrutturaBilancio.txt");
			conti = contiReader.readAll(reader);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, "Impossibile leggere la struttura del bilancio " + e.getMessage());
			return false;
		}
		
		Collection<Movimento> movimenti;
		try
		{
			FileInputStream inputStream = new FileInputStream("Bilancio.dat");
			movimenti = movimentiManager.leggiMovimenti(inputStream);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, "Impossibile leggere i movimenti " + e.getMessage());
			movimenti = new ArrayList<Movimento>();
		}

		bilancio = new BilancioFamiliare(conti, movimenti);
		return true;
	}
	
	@Override
	public Bilancio getConto(String nome)
	{
		return bilancio.getConto(nome);
	}
	
	@Override
	public Bilancio getBilancioFamiliare()
	{
		return bilancio;
	}
	
	public void showUI()
	{
		MainFrame view = new MainFrame();
		view.setController(this);
		view.setVisible(true);
	}

	@Override
	public Iterable<String> getConti()
	{
		ArrayList<String> nomi = new ArrayList<String>();
		for (Conto conto : bilancio.getConti())
		{
			nomi.add(conto.getName());
		}
		return nomi;
	}

	@Override
	public void openMovimenti()
	{
		MyMovimentoController controller = new MyMovimentoController(bilancio);
		controller.showUI();
	}
}
